Package: airGR
Type: Package
Title: Suite of GR Hydrological Models for Precipitation-Runoff
        Modelling
Version: 1.7.9
Date: 2026-07-08
Authors@R: c(
  person("Laurent", "Coron", role = c("aut", "trl"), comment = c(ORCID = "0000-0002-1503-6204")),
  person("Olivier", "Delaigue", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-7668-8468"), email = "airGR@inrae.fr"),
  person("Guillaume", "Thirel", role = c("aut", "ths"), comment = c(ORCID = "0000-0002-1444-1830")),
  person("David", "Dorchies", role = c("aut"), comment = c(ORCID = "0000-0002-6595-7984")),
  person("Charles", "Perrin", role = c("aut", "ths"), comment = c(ORCID = "0000-0001-8552-1881")),
  person("Claude", "Michel", role = c("aut", "ths")),
  person("Vazken", "Andréassian", role = c("ctb", "ths"), comment = c(ORCID = "0000-0001-7124-9303")),
  person("François", "Bourgin", role = c("ctb"), comment = c(ORCID = "0000-0002-2820-7260")),
  person("Pierre", "Brigode", role = c("ctb"), comment = c(ORCID = "0000-0001-8257-0741")),
  person("Nicolas", "Le Moine", role = c("ctb")),
  person("Thibaut", "Mathevet", role = c("ctb"), comment = c(ORCID = "0000-0002-4142-4454")),
  person("Safouane", "Mouelhi", role = c("ctb")),
  person("Ludovic", "Oudin", role = c("ctb"), comment = c(ORCID = "0000-0002-3712-0933")),
  person("Raji", "Pushpalatha", role = c("ctb")),
  person("Audrey", "Valéry", role = c("ctb"))
  )
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, stats, utils
Suggests: knitr, markdown, rmarkdown, caRamel, coda, DEoptim, FME,
        ggmcmc, Rmalschains, GGally, ggplot2, testthat, tibble
Description: Hydrological modelling tools developed at INRAE-Antony (HYCAR Research Unit, France). The package includes several conceptual rainfall-runoff models (GR4H, GR5H, GR4J, GR5J, GR6J, GR2M, GR1A) that can be applied either on a lumped or semi-distributed way. A snow accumulation and melt model (CemaNeige) and the associated functions for the calibration and evaluation of models are also included. Use help(airGR) for package description and references.
License: GPL-2
URL: https://hydrogr.github.io/airGR/
BugReports: https://forge.inrae.fr/hydroGR/airgrgalaxy/airgr/-/work_items
NeedsCompilation: yes
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Packaged: 2026-07-08 11:40:14 UTC; odelaigue
Author: Laurent Coron [aut, trl] (ORCID:
    <https://orcid.org/0000-0002-1503-6204>),
  Olivier Delaigue [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7668-8468>),
  Guillaume Thirel [aut, ths] (ORCID:
    <https://orcid.org/0000-0002-1444-1830>),
  David Dorchies [aut] (ORCID: <https://orcid.org/0000-0002-6595-7984>),
  Charles Perrin [aut, ths] (ORCID:
    <https://orcid.org/0000-0001-8552-1881>),
  Claude Michel [aut, ths],
  Vazken Andréassian [ctb, ths] (ORCID:
    <https://orcid.org/0000-0001-7124-9303>),
  François Bourgin [ctb] (ORCID: <https://orcid.org/0000-0002-2820-7260>),
  Pierre Brigode [ctb] (ORCID: <https://orcid.org/0000-0001-8257-0741>),
  Nicolas Le Moine [ctb],
  Thibaut Mathevet [ctb] (ORCID: <https://orcid.org/0000-0002-4142-4454>),
  Safouane Mouelhi [ctb],
  Ludovic Oudin [ctb] (ORCID: <https://orcid.org/0000-0002-3712-0933>),
  Raji Pushpalatha [ctb],
  Audrey Valéry [ctb]
Maintainer: Olivier Delaigue <airGR@inrae.fr>
Repository: CRAN
Date/Publication: 2026-07-08 12:50:02 UTC
