---
title: "Get Started with airGR"
author: "Guillaume Thirel, Olivier Delaigue, Laurent Coron"
bibliography: V00_airgr_ref.bib
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Get Started with airGR}
  %\VignetteEncoding{UTF-8}
---

# Introduction

**airGR** is a package that brings into the [**R software**](https://cran.r-project.org/) the hydrological modelling tools used and developed at the [Catchment Hydrology Research Group](https://webgr.inrae.fr/eng) at [INRAE (France)](https://www.inrae.fr/en), including the [**GR rainfall-runoff models**](https://webgr.inrae.fr/eng/tools/hydrological-models/) that can be applied either on a **lumped** or **semi-distributed** way. A snow accumulation and melt model ([**CemaNeige**](https://webgr.inrae.fr/eng/tools/hydrological-models/snow-model/)) and the associated functions for the calibration and evaluation of models are also included. Each model core is coded in **Fortran** to ensure low computational time. The other package functions (i.e. mainly the calibration algorithm and the efficiency criteria calculation) are coded in **R**. 


The **airGR** package has been designed to fulfill two major requirements: to facilitate the use by non-expert users and to allow flexibility regarding the addition of external criteria, models or calibration algorithms. The names of the functions and their arguments were chosen to this end. **airGR** also contains basics plotting facilities. 



Seven hydrological models and one snow melt and accumulation model are implemented in **airGR**. The hydrological models can be applied either on a lumped way or on a semi-distributed way (on sub-catchments). The snow model can either be used alone or with the daily or hourly hydrological models. Naturally each hydrological model can also be used alone.

The models can be called within **airGR** using the following functions: 

  * `RunModel_GR4H()`: four-parameter hourly lumped hydrological model [@mathevet_quels_2005] 
  * `RunModel_GR5H()`: five-parameter hourly lumped hydrological model [@ficchi_adaptive_2017; @ficchi_hydrological_2019] 
  * `RunModel_GR4J()`: four-parameter daily lumped hydrological model [@perrin_improvement_2003] 
  * `RunModel_GR5J()`: five-parameter daily lumped hydrological model [@le_moine_bassin_2008] 
  * `RunModel_GR6J()`: six-parameter daily lumped hydrological model [@pushpalatha_downward_2011] 
  * `RunModel_GR2M()`: two-parameter monthly lumped hydrological model [@mouelhi_vers_2003; @mouelhi_stepwise_2006] 
  * `RunModel_GR1A()`: one-parameter yearly lumped hydrological model [@mouelhi_vers_2003; @mouelhi_linking_2006] 
  * `RunModel_CemaNeige()`: two-parameter degree-day snowmelt and accumulation model [@valery_as_2014; @riboust_revisiting_2019] 
  * `RunModel_CemaNeigeGR4H()`: combined use of **GR4H** and **CemaNeige** 
  * `RunModel_CemaNeigeGR5H()`: combined use of **GR5H** and **CemaNeige** 
  * `RunModel_CemaNeigeGR4J()`: combined use of **GR4J** and **CemaNeige** 
  * `RunModel_CemaNeigeGR5J()`: combined use of **GR5J** and **CemaNeige** 
  * `RunModel_CemaNeigeGR6J()`: combined use of **GR6J** and **CemaNeige** 

The [**GRP**](https://webgr.inrae.fr/eng/tools/hydrological-models/hydrological-forecasting-model-grp/) forecasting model and the [**Otamin**](https://webgr.inrae.fr/eng/tools/software/otamin) predictive uncertainty tool are not available in **airGR**.

In this vignette, we show how to prepare and run a calibration and a simulation with airGR hydrological models.


# Loading data

In the following example, we use a data sample contained in the package. For real applications, the user has to import its data into **R** and to prepare it with an adequate data.frame format as described below.


First, it is necessary to load the **airGR** package:

```{r, warning=FALSE}
library(airGR)
```

Below is presented an example of a `data.frame` of daily hydrometeorological observations time series for a fictional catchment included in the **airGR** package that contains:

  * *DatesR*: dates in the POSIXt format 
  * *P*: average precipitation [mm/day]
  * *T*: catchment average air temperature [&#8451;]
  * *E*: catchment average potential evapotranspiration [mm/day]
  * *Qls*: outlet discharge [l/s]
  * *Qmm*: outlet discharge [mm/day]
  
```{r}
data(L0123001)
summary(BasinObs, digits = 2)
```
The usual functions (e.g. `read.table()`) can be used to load real-case data sets.



# Preparation of functions inputs

To run a model, the functions of the **airGR** package (e.g. the models, calibration and criteria calculation functions) require data and options with specific formats.

To facilitate the use of the package, there are several functions dedicated to the creation of these objects:

  * `CreateInputsModel()`: prepares the inputs for the different hydrological models (times series of dates, precipitation, observed discharge, etc.)
  * `CreateRunOptions()`: prepares the options for the hydrological model run (warm up period, calibration period, etc.)
  * `CreateInputsCrit()`: prepares the options in order to compute the efficiency criterion (choice of the criterion, choice of the transformation on discharge: "log", "sqrt", etc.)
  * `CreateCalibOptions()`: prepares the options for the hydrological model calibration algorithm (choice of parameters to optimize, predefined values for uncalibrated parameters, etc.)


## InputsModel object

To run a GR hydrological model or CemaNeige, the user has to prepare the input data with the `CreateInputsModel()` function.
As arguments, this function needs the function name corresponding to the model the user wants to run, a vector of dates, a vector of precipitation and a vector of potential evapotranspiration.

In the example below, we already have the potential evapotranspiration. If the user does not have these data, it is possible to compute it with the [Oudin's formula](http://dx.doi.org/10.1016/j.jhydrol.2004.08.026) with the `PE_Oudin()` function (this function only needs Julian days, daily average air temperature and latitude).

Missing values (`NA`) of precipitation (or potential evapotranspiration) are **not allowed**.


```{r}
InputsModel <- CreateInputsModel(FUN_MOD = RunModel_GR4J, DatesR = BasinObs$DatesR,
                                 Precip = BasinObs$P, PotEvap = BasinObs$E)
str(InputsModel)
```


## RunOptions object

The `CreateRunOptions()` function allows to prepare the options required to the `RunModel*()` functions, which are the actual models functions.

The user must at least define the following arguments:

  * `FUN_MOD`: the name of the model function to run
  * `InputsModel`: the associated input data
  * `IndPeriod_Run`: the period on which the model is run
  
To select a period for which the user wants to run the model, select the corresponding indexes for different time periods (not the POSIXt dates), as follows:

```{r}
Ind_Run <- seq(which(format(BasinObs$DatesR, format = "%Y-%m-%d") == "1990-01-01"), 
               which(format(BasinObs$DatesR, format = "%Y-%m-%d") == "1999-12-31"))
str(Ind_Run)
```

The initialization of hydrological models is of the utmost importance. Indeed, an inaccurate initialization causes poor quality discharge simulations during the earliest stages of the running period. For example, in the GR models, by default, the production and the routing store levels store level are respectively set to 30 % and 50 % of their capacity, which may be far from their ideal value. Two solutions are offered to accurately initialize the GR models in **airGR**: manually predefining the initial states (e.g. from a previous run) or running the models during a warm up period before the actual running period. It is generally advised to set up this warm up period to be equal or superior to one year. 

As a consequence, it is possible to define in `CreateRunOptions()` the following arguments:

  * `IniStates`: the initial states of the 2 unit hydrographs (20 + 40 = 60 units)
  * `IniResLevels`: the initial levels of the production and routing stores
  * `IndPeriod_WarmUp`: the warm up period used to run the model, to be defined in the same format as `IndPeriod_Run`


```{r}
RunOptions <- CreateRunOptions(FUN_MOD = RunModel_GR4J,
                               InputsModel = InputsModel, IndPeriod_Run = Ind_Run,
                               IniStates = NULL, IniResLevels = NULL, IndPeriod_WarmUp = NULL)
str(RunOptions)
```
The `CreateRunOptions()` function returns warnings if the default initialization options are used:

  * `IniStates` and `IniResLevels` are automatically set to initialize all the model states at 0, except for the production and routing stores, which are initialized at respectively 30 % and 50 % of their capacity
  * `IndPeriod_WarmUp` default setting ensures a one-year warm up using the time steps preceding the `IndPeriod_Run`, if available  


## InputsCrit object

The `CreateInputsCrit()` function allows to prepare the input in order to calculate a criterion. It is possible to define the following arguments:

  * `FUN_CRIT`: the name of the error criterion function (the available functions are introduced later on)
  * `InputsModel`: the inputs of the hydrological model previously prepared by the `CreateInputsModel()` function
  * `RunOptions`: the options of the hydrological model previously prepared by the `CreateRunOptions()` function 
  * `VarObs`: the name of the considered variable (by default `"Q"` for the discharge)
  * `Obs`: the observed variable time serie (e.g. the discharge expressed in *mm/time step*)

Missing values (`NA`) are **allowed** for observed discharge.

It is possible to compute a composite criterion (e.g. the average between NSE computed on discharge and NSE computed on log of discharge). In this case, users have to provide lists to the following arguments (some of the are optional): `FUN_CRIT`, `Obs`, `VarObs`, `BoolCrit`, `transfo`, `Weights.` 

```{r}
InputsCrit <- CreateInputsCrit(FUN_CRIT = ErrorCrit_NSE, InputsModel = InputsModel, 
                               RunOptions = RunOptions, VarObs = "Q", Obs = BasinObs$Qmm[Ind_Run])
str(InputsCrit)
```


## CalibOptions object

Before using the automatic calibration tool, the user needs to prepare the calibration options with the `CreateCalibOptions()` function. For that, it is necessary to define the following arguments:

  * `FUN_MOD`: the name of the model function
  * `FUN_CALIB`: the name of the calibration algorithm

```{r}
CalibOptions <- CreateCalibOptions(FUN_MOD = RunModel_GR4J, FUN_CALIB = Calibration_Michel)
str(CalibOptions)
```


# Criteria

The evaluation of the quality of a simulation is estimated through the calculation of criteria. These criteria can be used both as objective-functions during the calibration of the model, or as a measure for evaluating its performance on a control period.


The package offers the possibility to use different criteria:

  * `ErrorCrit_RMSE()`: Root mean square error (RMSE)
  * `ErrorCrit_NSE()`: Nash-Sutcliffe model efficiency coefficient (NSE)
  * `ErrorCrit_KGE()`: Kling-Gupta efficiency criterion (KGE)
  * `ErrorCrit_KGE2()`: modified Kling-Gupta efficiency criterion (KGE')
  
It is also possible to create user-defined criteria. For doing that, it is only necessary to define the function in **R** following the same syntax as the criteria functions included in **airGR**.
  
  
# Calibration


The objective of the calibration algorithm is to identify the model parameters: by comparing the model outputs with observed data, this algorithm determines the combination of parameters that represents the best the behavior of the watershed.


In the **airGR** package, a function called `Calibration_Michel()` is implemented. This functions allows running a calibration with the package models.
The calibration algorithm optimizes the error criterion selected as objective-function. This algorithm works in two steps:

  1. a screening of the parameters space is performed using either a rough predefined grid or a user-defined list of parameter sets
  2. a simple steepest descent local search algorithm is performed from the best set of parameters found at the first step

```{r}
OutputsCalib <- Calibration_Michel(InputsModel = InputsModel, RunOptions = RunOptions,
                                   InputsCrit = InputsCrit, CalibOptions = CalibOptions,
                                   FUN_MOD = RunModel_GR4J)
Param <- OutputsCalib$ParamFinalR
Param
```

The `Calibration_Michel()` function is the only one implemented in the **airGR** package to calibrate the model, but the user can implement its own calibration function. Two vignettes explain how it can be done ([2.1 Plugging in new calibration](V02.1_param_optim.html) and [2.2 MCMC parameter estimation](V02.2_param_mcmc.html)).

The `Calibration_Michel()` function returns a vector with the parameters of the chosen model, which means that the number of values can differ depending on the model that is used. It is possible to use the `Calibration_Michel()` function with user-implemented hydrological models. 



# Control

This step assesses the predictive capacity of the model. Control is defined as the estimation of the accuracy of the model on data sets that are not used in its construction, and in particular its calibration.
The classical way to perform a control is to keep data from a period separated from the calibration period. If possible, this control period should correspond to climatic situations that differ from those of the calibration period in order to better point out the qualities and weaknesses of the model. This exercise is necessary for assessing the robustness of the model, that is to say its ability to keep stable performances outside of the calibration conditions.

Performing a model control with **airGR** is similar to running a simulation (see below), followed by the computation of one or several performance criteria.



# Simulation

## Simulation run

To run a model, the user has to use the `RunModel*()` functions (`InputsModel`, `RunOptions` and parameters).
All the data needed have already been prepared in the previous steps defined in this guide.

```{r}
OutputsModel <- RunModel_GR4J(InputsModel = InputsModel, RunOptions = RunOptions, Param = Param)
str(OutputsModel)
```


## Results preview

Although it is possible for the user to design its own graphics from the outputs of the `RunModel*()` functions, the **airGR** package offers the possibility to make use of the `plot()` function. This function returns a dashboard of results including various graphs (depending on the model used): 

  * time series of total precipitation and simulated discharge (and observed discharge if provided)
  * interannual average daily simulated discharge (and daily observed discharge if provided) and interannual average monthly precipitation 
  * cumulative frequency plot for simulated discharge (and for observed discharge if provided)
  * correlation plot between simulated and observed discharge (if observed discharge provided)
  
```{r,eval=F}
plot(OutputsModel, Qobs = BasinObs$Qmm[Ind_Run])
```

Moreover, if the CemaNeige model is used, the air temperature and the simulated snowpack water equivalent time series are plotted. 


## Efficiency criterion

To evaluate the efficiency of the model, it is possible to use the same criterion as defined at the calibration step or to use another criterion.

```{r}
OutputsCrit <- ErrorCrit_NSE(InputsCrit = InputsCrit, OutputsModel = OutputsModel)
str(OutputsCrit)
```
```{r}
OutputsCrit <- ErrorCrit_KGE(InputsCrit = InputsCrit, OutputsModel = OutputsModel)
str(OutputsCrit)
```


## Features diagram

The figure below presents the implementation flow of the **airGR** functions.
<br><br>
<img src="fig/airGR_features_diagram.svg" alt="airGR features diagram" width=75% />


# References
